#include <stdint.h>
#include <stdio.h>

#include "sdkconfig.h"

/* FreeRTOS */
#include "freertos/FreeRTOS.h"
#include "freertos/event_groups.h"
#include "freertos/task.h"

/* IDF */
#include "driver/rmt_tx.h"
#include "esp_log.h"

#define LOG_TAG "LED"

#define LED_NP_PIN (8)

static rmt_encoder_handle_t s_led_enc;
static rmt_channel_handle_t s_led_chan;

int app_led_init(void) {
    esp_err_t ret = ESP_OK;

    rmt_tx_channel_config_t led_chan_config = {
        .clk_src           = RMT_CLK_SRC_DEFAULT,
        .gpio_num          = LED_NP_PIN,
        .mem_block_symbols = 64,
        .resolution_hz     = 10000000UL,
        .trans_queue_depth = 4,
    };

    ret = rmt_new_tx_channel(&led_chan_config, &s_led_chan);
    if(ret != ESP_OK) {
        return -1;
    }

    rmt_bytes_encoder_config_t led_enc_config = {
        .bit0 =
            {
                .level0    = 1,
                .duration0 = 3,
                .level1    = 0,
                .duration1 = 9,
            },
        .bit1 =
            {
                .level0    = 1,
                .duration0 = 9,
                .level1    = 0,
                .duration1 = 3,
            },
    };

    ret = rmt_new_bytes_encoder(&led_enc_config, &s_led_enc);
    if (ret != ESP_OK) {
        ESP_LOGE(LOG_TAG, "Failed to create LED encoder.");
        return -2;
    }

    return 0;
}
